
/* Ekiga -- A VoIP and Video-Conferencing application
 * Copyright (C) 2000-2006 Damien Sandras
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 *
 * Ekiga is licensed under the GPL license and as a special exception,
 * you have permission to link or otherwise combine this program with the
 * programs OPAL, OpenH323 and PWLIB, and distribute the combination,
 * without applying the requirements of the GNU GPL to the OPAL, OpenH323
 * and PWLIB programs, as long as you do follow the requirements of the
 * GNU GPL for all the rest of the software thus combined.
 */


/*
 *                         gm-call-handler.c - description
 *                         -------------------------------
 *   begin                : Son Mar 18 2007
 *   copyright            : (C) 2007 by Jan Schampera
 *   description          : the GmCallHandler object
 */

/*!\file gm-call-handler.c
 * \brief the GmCallHandler object implementation
 * \author Jan Schampera
 * \date 2007
 * \ingroup call_handling
 */

#include <glib.h>

#include "gm-call-handler.h"

/* variables, constants, vodoo */

/* just for debug */
const char * call_direction [] = { "in", "out", "transfer"};

enum {
  SIG_CALL_STATUS_CHANGED,
  SIG__LAST
};

static guint gm_call_handler_signals [SIG__LAST] = { 0 };

struct _GmCallHandlerPrivate {
  GSList* in_handlers;   /*!< handler list for IN handlers */
  GSList* out_handlers;  /*!< handler list for OUT handlers */
  GSList* xfer_handlers; /*!< handler list for XFER handlers */
};

typedef struct _Handler {
  GmCallHandlerFunc handler_func;
  GmCallDirection direction;
  gpointer user_data;
} Handler;

/* prototyping */

/* misc */
static GSList* gm_call_handler_get_handler_list (GmCallHandler*,
						 GmCallDirection);
static void gm_call_handler_set_handler_list (GmCallHandler*,
					      GmCallDirection,
					      GSList*);


/* the handler struct management */
static Handler* handler_new (GmCallHandlerFunc, GmCallDirection, gpointer);
static void handler_destroy (Handler*);


/* callinfo block management code: copy, delete, ... */
static GmEndpointIdentifier* ep_id_copy (GmEndpointIdentifier*);
static void ep_id_delete (GmEndpointIdentifier*);
static GmCallInfo* call_info_copy (GmCallInfo*);
static void call_info_delete (GmCallInfo*);


/* gobject stuff */
static void gm_call_handler_class_init (GmCallHandlerClass*);
static void gm_call_handler_init (GmCallHandler*);


/* implementation */


/* callinfo block management code: copy, delete, ... */
static GmEndpointIdentifier*
ep_id_copy (GmEndpointIdentifier* ep_id) {
  GmEndpointIdentifier* new_ep_id = NULL;

  if (!ep_id)
    return (GmEndpointIdentifier*) NULL;

  new_ep_id = g_new (GmEndpointIdentifier, 1);

  new_ep_id->uri = g_strdup (ep_id->uri);
  new_ep_id->user_agent = g_strdup (ep_id->user_agent);
  new_ep_id->name = g_strdup (ep_id->name);

  return new_ep_id;
}


static void
ep_id_delete (GmEndpointIdentifier* ep_id) {
  g_return_if_fail (ep_id != NULL);

  g_free (ep_id->uri);
  g_free (ep_id->user_agent);
  g_free (ep_id->name);

  g_free (ep_id);

  g_nullify_pointer ((gpointer) &ep_id);
}


static GmCallInfo*
call_info_copy (GmCallInfo* call_info) {
  GmCallInfo* new_call_info = NULL;

  g_return_val_if_fail (call_info != NULL, NULL);

  new_call_info = g_new (GmCallInfo, 1);

  new_call_info->from = ep_id_copy (call_info->from);
  new_call_info->to = ep_id_copy (call_info->to);
  new_call_info->transferrer = ep_id_copy (call_info->transferrer);
  new_call_info->token = g_strdup (call_info->token);
  new_call_info->direction = call_info->direction;
  new_call_info->protocol = call_info->protocol;
  new_call_info->results = call_info->results;

  return new_call_info;
}


static void
call_info_delete (GmCallInfo* call_info) {
  g_return_if_fail (call_info != NULL);

  if (call_info->from)
    ep_id_delete (call_info->from);
  if (call_info->to)
    ep_id_delete (call_info->to);
  if (call_info->transferrer)
    ep_id_delete (call_info->transferrer);
  g_free (call_info->token);

  g_free (call_info);
  
  g_nullify_pointer ((gpointer) &call_info);
}


GmCallHandler*
gm_call_handler_new (void) {
  return GM_CALL_HANDLER (g_object_new (GM_CALL_HANDLER_TYPE, NULL));
}


static void gm_call_handler_class_init (GmCallHandlerClass *klass) {
  /* FIXME define the signals */
}

static void gm_call_handler_init (GmCallHandler *self) {
 
  self->priv = g_new (GmCallHandlerPrivate, 1);

  gm_call_handler_set_handler_list (self, GM_CALL_IN, NULL);
  gm_call_handler_set_handler_list (self, GM_CALL_OUT, NULL);
  gm_call_handler_set_handler_list (self, GM_CALL_TRANSFER, NULL);
}


static GSList*
gm_call_handler_get_handler_list (GmCallHandler *self,
				  GmCallDirection direction) {
  GmCallHandlerPrivate *priv = NULL;

  g_return_val_if_fail (self != NULL, (GSList*) NULL);
  g_return_val_if_fail (GM_IS_CALL_HANDLER (self), (GSList*) NULL);

  priv = self->priv;

  switch (direction) {
  case GM_CALL_IN:
    return priv->in_handlers;
    break;
  case GM_CALL_OUT:
    return priv->out_handlers;
    break;
  case GM_CALL_TRANSFER:
    return priv->xfer_handlers;
    break;
  default:
    return (GSList*) NULL;
    break;
  }
}


static void
gm_call_handler_set_handler_list (GmCallHandler *self,
				  GmCallDirection direction,
				  GSList *list) {
  GmCallHandlerPrivate *priv = NULL;

  g_return_if_fail (self != NULL);
  g_return_if_fail (GM_IS_CALL_HANDLER (self));

  priv = self->priv;

  switch (direction) {
  case GM_CALL_IN:
    priv->in_handlers = list;
    break;
  case GM_CALL_OUT:
    priv->out_handlers = list;
    break;
  case GM_CALL_TRANSFER:
    priv->xfer_handlers = list;
    break;
  }
}


gpointer gm_call_handler_register_handler (GmCallHandler *self,
					   GmCallDirection direction,
					   guint priority,
					   GmCallHandlerFunc func,
					   gpointer user_data) {
  guint list_length = 0;
  gint list_prio_pos = 0;

  GSList *list = NULL;
  GmCallHandlerPrivate *priv = NULL;
  Handler *handler = NULL;

  g_return_val_if_fail (GM_IS_CALL_HANDLER (self), NULL);
  g_return_val_if_fail (((priority > 0) || (priority <= 100)), NULL);
  g_return_val_if_fail (func != NULL, NULL);

  g_debug ("Registering a handler with prio=%d for calls=%s",
	   priority, call_direction [direction]);

  priv = self->priv;

  handler = handler_new (func, direction, user_data);

  list = gm_call_handler_get_handler_list (self, direction);

  /* sort the handler into the list according to the requested priority
   * HIGHEST will always result in the very first element used (list prepend)
   * LOWEST will always result in the very last element used (list append) */

  /* FIXME the priorizing calc SUCKS!!! */
  switch (priority) {
  case GM_CALL_HANDLER_PRIO_HIGHEST:
    list = g_slist_prepend (list, (gpointer) handler);
    break;
  case GM_CALL_HANDLER_PRIO_LOWEST:
    list = g_slist_append (list, (gpointer) handler);
    break;
  default:
    list_length = g_slist_length (list);
    list_prio_pos = (int) (((float) list_length / 100.0) * (float) priority);
    list = g_slist_insert (list, (gpointer) handler, list_prio_pos);
    break;
  }

  gm_call_handler_set_handler_list (self, direction, list);

  return handler;
}


void gm_call_handler_unregister_handler (GmCallHandler *self,
					 gpointer handler_id) {
  Handler *handler = NULL;
  GSList *list = NULL;
  GSList *list_iter = NULL;
  GmCallHandlerPrivate *priv = NULL;
  GmCallDirection direction;

  g_return_if_fail (self != NULL);
  g_return_if_fail (GM_IS_CALL_HANDLER (self));
  g_return_if_fail (handler_id != NULL);

  priv = self->priv;
  handler = (Handler*) handler_id;

  direction = handler->direction;

  list = gm_call_handler_get_handler_list (self, direction);

  list = g_slist_remove_all (list, (gconstpointer) handler_id);

  gm_call_handler_set_handler_list (self, direction, list);

  handler_destroy (handler);
}


static Handler *handler_new (const GmCallHandlerFunc func,
			     GmCallDirection direction,
			     gpointer user_data) {
  Handler *handler = NULL;

  g_return_val_if_fail (func != NULL, NULL);

  handler = g_new (Handler, 1);

  handler->handler_func = func;
  handler->direction = direction;
  handler->user_data = user_data;

  return handler;
}


void handler_destroy (Handler* handler) {
  g_return_if_fail (handler != NULL);

  g_free (handler);
}


void gm_call_handler_handle (GmCallHandler *self,
                             GmCallInfo *call_info) {
  /* Handling calls:
   * - send an informational signal that a call happened FIXME
   * - process the list of handlers relevant for that call FIXME INCOMLETE
   * - send an informational signal FIXME
   */
  GSList *list = NULL;
  GmCallHandlerPrivate *priv = NULL;
  GmCallHandlerFunc handler_func = NULL;
  Handler *handler = NULL;
  GmCallInfo* tmp_call_info = NULL;
  gboolean was_handled = FALSE;

  g_return_if_fail (self != NULL);
  g_return_if_fail (call_info != NULL);

  priv = self->priv;

  g_debug ("Attempting to handle the call: %s", call_info->token);
  g_debug ("Call direction: %s", call_direction[call_info->direction]);

  list = gm_call_handler_get_handler_list (self, call_info->direction);

  /* the loop that processes the handler list */
  for (;
       list != NULL;
       list = g_slist_next (list)) {

    handler = (Handler*) list->data;
    if (handler && handler->handler_func) {

      /* the copy is to make sure the handler can't destroy useful data */
      tmp_call_info = call_info_copy (call_info);
      tmp_call_info->results = GM_CALL_RESULT_NONE;

      g_debug ("gm_call_handler_handle(): calling a handler");
      if (handler->handler_func (self, tmp_call_info, (gpointer) handler))
	{ /* handled */

	  g_debug ("gm_call_handler_handle(): handler handled the call, STOP");
	  list = NULL; /* prematurely exit the loop */
	  was_handled = TRUE;
	  if (tmp_call_info->results != GM_CALL_RESULT_NONE)
	    call_info->results = tmp_call_info->results;

	} else { /* not handled */

	  g_debug ("gm_call_handler_handle(): handler didn't handle the call, NEXT");

	}

      call_info_delete (tmp_call_info);
    }

    if (was_handled) {
      /* inspect call_info->results ... and react */
    }

    /* FIXME something like (but with a copy) */
    /*   gm_call_info_signal (self, call_info); */
  }
}



void gm_call_handler_set_uniq_token (GmCallHandler *self,
				     GmCallInfo *call_info) {
  g_return_if_fail (self != NULL);
  g_return_if_fail (call_info != NULL);

  /* what a hack */
  call_info->token = g_strdup_printf ("%04X-%04X-%04X-EKIGA_2.1.0",
				      g_random_int_range (0, 65535),
				      g_random_int_range (0, 65535),
				      g_random_int_range (0, 65535));
}


GmCallInfo* gm_call_handler_new_call_info () {
  GmCallInfo *call_info = NULL;

  call_info = g_new (GmCallInfo, 1);

  /* this isn't meant to set useful default, it's meant
   * to set *something*! */
  call_info->token = NULL;
  call_info->direction = GM_CALL_IN;
  call_info->protocol = GM_CALL_PROTOCOL_SIP;
  call_info->from = NULL;
  call_info->to = NULL;
  call_info->transferrer = NULL;

  return call_info;
}


GType
gm_call_handler_get_type () {

  static GType gm_call_handler_type = 0;

  if (!gm_call_handler_type) {
    static const GTypeInfo gm_call_handler_info =
      {
	sizeof (GmCallHandlerClass),
	NULL,
	NULL,
	(GClassInitFunc) gm_call_handler_class_init,
	NULL,
	NULL,
	sizeof (GmCallHandler),
	0,
	(GInstanceInitFunc) gm_call_handler_init
      };
    gm_call_handler_type =
      g_type_register_static (G_TYPE_OBJECT,
			      "GmCallHandler",
			      &gm_call_handler_info, (GTypeFlags) 0);
  }

  return gm_call_handler_type;
}

