
/* Ekiga -- A VoIP and Video-Conferencing application
 * Copyright (C) 2000-2006 Damien Sandras
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 *
 * Ekiga is licensed under the GPL license and as a special exception,
 * you have permission to link or otherwise combine this program with the
 * programs OPAL, OpenH323 and PWLIB, and distribute the combination,
 * without applying the requirements of the GNU GPL to the OPAL, OpenH323
 * and PWLIB programs, as long as you do follow the requirements of the
 * GNU GPL for all the rest of the software thus combined.
 */


/*
 *                         gm-call-handler.h - description
 *                         --------------------------------
 *   begin                : Sun Mar 18 2007
 *   copyright            : (C) 2007 by Jan Schampera
 *   description          : Header file for the call handler object
 */

/*!\defgroup call_handling Call handling and call processing
 */

/*!\file gm-call-handler.h
 * \brief Header file for the GmCallHandler
 * \author Jan Schampera <jan.schampera@unix.net>
 * \date 2007
 * \ingroup call_handling
 */

/*!\defgroup GmCallHandler Implement the call handling in Ekiga
 */

#ifndef GM_CALL_HANDLER_H__
#define GM_CALL_HANDLER_H__

#include <glib.h>
#include <glib-object.h>

G_BEGIN_DECLS

#define GM_CALL_HANDLER_TYPE             (gm_call_handler_get_type ())
/*!< returns the GType of GmCallHandler */
#define GM_CALL_HANDLER(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), GM_CALL_HANDLER_TYPE, GmCallHandler))
/*!< casts a pointer to GmCallHandler if the type matches */
#define GM_CALL_HANDLER_CLASS(klass)     ((G_TYPE_CHECK_CLASS_CAST ((klass), GM_CALL_HANDLER_TYPE, GmCallHandlerClass)))
/*!< casts a pointer to GmCallHandlerClass if the type matches */
#define GM_IS_CALL_HANDLER(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GM_CALL_HANDLER_TYPE))
/*!< checks if the given pointer points to a GmCallHandler* instance */
#define GM_IS_CALL_HANDLER_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), GM_CALL_HANDLER_TYPE))
/*!< checks if the given pointer points to a GmCallHandlerClass* class */

typedef enum {
  GM_CALL_IN = 0,
  GM_CALL_OUT,
  GM_CALL_TRANSFER
} GmCallDirection;

/*!
 * \brief the used protocol (used to describe a call)
 */
typedef enum {
  GM_CALL_PROTOCOL_H323 = 0, /*!< the H.323 protocol */
  GM_CALL_PROTOCOL_SIP       /*!< the SIP protocol */
} GmCallProtocol;

enum {
  GM_CALL_HANDLER_PRIO_HIGHEST = 0,  /*!< highest priority, handler is prepended */
  GM_CALL_HANDLER_PRIO_NORMAL = 50,  /*!< normal priority, calculated to the middle */
  GM_CALL_HANDLER_PRIO_DEFAULT = 75, /*!< default priority, after the middle */
  GM_CALL_HANDLER_PRIO_LOWEST = 100  /*!< lowest priority, handler is appended */
};

typedef enum {
  GM_CALL_RESULT_NONE         = 0,
  GM_CALL_RESULT_ACTIVATING   = 1 << 0,
  GM_CALL_RESULT_TERMINATING  = 1 << 1,
  GM_CALL_RESULT_TRANSFERRING = 1 << 2,
  GM_CALL_RESULT_ACCEPTED     = 1 << 3,
  GM_CALL_RESULT_REJECTED     = 1 << 4
} GmCallResults;


typedef struct _GmCallHandler GmCallHandler;

typedef struct _GmCallHandlerClass GmCallHandlerClass;

typedef struct _GmCallHandlerPrivate GmCallHandlerPrivate;


typedef struct _GmEndpointIdentifier {
  gchar* uri;
  /*!< the protocol specific URI of the phony engine */

  gchar* user_agent;
  /*!< the name of the software/user agent used */

  gchar* name;
  /*!< the name of the user or some identifier */
} GmEndpointIdentifier;

typedef struct _GmCallInfo {
  gchar* token;
  /*!< the call token, an uniq identifier for a call */

  GmCallDirection direction;
  /*!< the direction of the call */

  GmCallProtocol protocol;
  /*!< the used protocol */

  GmCallResults results;
  /*!< results (informational) set by a handler (accepted, rejected, ...) */

  GmEndpointIdentifier* from;
  /*!< source of the call */

  GmEndpointIdentifier* to;
  /*!< target of the call */

  GmEndpointIdentifier* transferrer;
  /*!< transferrer in case of call transferring */
} GmCallInfo;

/* model a call handler function */
typedef gboolean (*GmCallHandlerFunc) (GmCallHandler* call_handler,
				       GmCallInfo* call_info,
				       gpointer handler_id);


/* the object and class */

struct _GmCallHandler {
  GObject parent; /*< the parent instance, the GObject */

  GmCallHandlerPrivate *priv; /*< private internal data */
};

struct _GmCallHandlerClass {
  GObjectClass parent_class; /*!< the parent class, the GObjectClass */

  void (* call_status_changed) (GmCallHandler* call_handler,
				GmCallInfo* call_info);
  /*!< prototype for the signal handler of call-status-change */
};



/* the API */



/*!
 * \brief Creates a new GmCallHandler instance and returns it.
 *
 * \returns A fresh GmCallHandler object pointer.
 *
 * \see GmCallHandler
 */
GmCallHandler* gm_call_handler_new (void);

/*!
 * \brief Returns the GType of GmCallHandler.
 *
 * Usually used by the macros or by GLib/GTK itself. Happiness for GTK/GLib.
 *
 * \returns The runtime GType of GmCallHandler.
 *
 * \see GM_CALL_HANDLER_TYPE
 * \see GM_CALL_HANDLER
 * \see GM_CALL_HANDLER_CLASS
 * \see GM_IS_CALL_HANDLER
 * \see GM_IS_CALL_HANDLER_CLASS
 */
GType gm_call_handler_get_type (void);


/*!
 * \brief Register a new handler for a given direction with a given priority.
 *
 * \param call_handler A valid call handler instance, must not be NULL
 * \param direction To select one of the chains to put the handler in
 * \param priority The priority the handler gets in the chain
 * \param handler_func A pointer to a GmCallHandlerFunc
 * \param user_data A pointer that is handled through to the handler function
 *
 * When a handler function is registered, it is simply added to a sequential
 * list of handlers. The exact position in that list depends on the
 * priority given. The pointer returned can be used to identify the handler
 * in a gm_call_handler_unregister_handler request.
 *
 * \returns A pointer to internal data which is used to identify the handler
 *
 * \see gm_call_handler_unregister_handler
 */
gpointer gm_call_handler_register_handler (GmCallHandler *,
					   GmCallDirection,
					   guint,
					   GmCallHandlerFunc,
					   gpointer);


/*!
 * \brief Unregister an existing handler.
 *
 * \param call_handler A valid call handler instance, must not be NULL
 * \param handler_id A valid handler ID
 *
 * This function simply removes the handler from the lost, all needed is
 * the identify pointer to find the element. That pointer previously was
 * returned by gm_call_handler_register_handler, also it is part of the
 * parameter list of every handler function.
 *
 * \warning Currently the function is implemented but empty!
 *
 * \see gm_call_handler_register_handler
 */
void gm_call_handler_unregister_handler (GmCallHandler *,
					 gpointer);


/*!
 * \brief Requests to handle the described call.
 *
 * \param call_handler A valid call handler instance, must not be NULL
 * \param call_info The GmCallInfo block to describe the call
 *
 * The call is handled by seducing the given GmCallInfo block, hooking to
 * the right chain, and call the handlers one-by-one. The GmCallInfo block
 * MUST contain useful data, to make the GmCallHandler work right, at least
 * a call token and a direction info.
 */
void gm_call_handler_handle (GmCallHandler *,
			     GmCallInfo *);


/*!
 * \brief Sets a call token (uniq ID string) in a GmCallInfo block.
 *
 * \param call_handler A valid call handler instance, must not be NULL
 * \param call_info The GmCallInfo block to change
 *
 * Please always use that function to store a call token into the
 * GmCallInfo block. One reason is, that the format may change from
 * time to time, the other reason is, that only the GmCallHandler
 * knows if that token is uniq or not.
 */
void gm_call_handler_set_uniq_token (GmCallHandler *,
                                     GmCallInfo *);

/*!
 * \brief creates an empty GmCallInfo block
 *
 * The block is created using some default, not meant as usable default, more
 * meant to reduce randomness in content.
 *
 * \returns A pointer to an empty GmCallInfo block
 */
GmCallInfo* gm_call_handler_new_call_info (void);


G_END_DECLS

#endif /* GM_CALL_HANDLER_H__ */

